from email.policy import default
import click
from datetime import date, datetime, timedelta
import jurn.utils as u
import pprint

@click.group()
@click.option("--early-end", "-ee", type=int, help="Stop jurn from executing if this amount of time has not passed since the last journal entry. Useful to supress jurn prompting too frequently if adding to a .bashrc file or schedule.")
@click.option("--db-path", "-dp", help="The folder path for jurn's sqlite3 database. Defaults to ~/.jurn/ .", default="~/.jurn/")
@click.option("--db-filename", "-df", help="The filename for jurn's sqlite3 database. Defaults to jurn.db .", default="jurn.db")
@click.pass_context
def cli(ctx,early_end,db_path,db_filename):
    ctx.ensure_object(dict)
    ctx.obj['DB_CONNECTION'] = u.init_db(db_path,db_filename)

    # determine if the prompt should be skipped based on the --early-end parameter and the last entry's timestamp.
    last_entry_timestamp=u.last_entry_timestamp(ctx.obj['DB_CONNECTION'])

    if early_end==None:
        pass
    else:
        delay_until_timestamp = last_entry_timestamp + timedelta(minutes=early_end)
        if delay_until_timestamp < datetime.utcnow():
            pass
        else:
            exit()




@cli.command()
@click.option("--message", "-m", prompt=True, prompt_required=True, required=True, is_flag=False, help="Message to save to the journal.")
@click.option("--tag", "-t", prompt=True, prompt_required=False, default="", type=u.DistinctTags(), help="Tags to use for your journal entry. Hierarchies can be denoted with hashtags, eg. parent-cateogry#sub-category#child-category")
@click.pass_context
def log(ctx,message,tag):
    """Adds a journal entry to the database."""

    with ctx.obj['DB_CONNECTION']:
        ctx.obj['DB_CONNECTION'].execute("INSERT INTO entry (entry,tag) VALUES(?,?)",(message,tag,))

@cli.command()
@click.option("--duration", "-d", type=click.Choice(['day','week','month','year'], case_sensitive=False), help="Inclusive time period to print journal entries for.")
@click.option('--date-start', "-ds", type=click.DateTime(formats=["%Y-%m-%d"]), help="Inclusive start date range.")
@click.option('--date-end', "-de", type=click.DateTime(formats=["%Y-%m-%d"]), help="Inclusive end date range.")
@click.pass_context
def print(ctx,duration,date_start,date_end):
    """Outputs journal entries to the terminal for a specified date range (defaults to today)."""
    
    (date_start,date_end)=u.calculate_date_range(duration,date_start,date_end)
    entries = u.retrieve_entries(date_start,date_end,ctx.obj['DB_CONNECTION'])
    
    previous_tags = ['']
    
    if str(date_start)[:10] == str(date_end)[:10]:
        click.echo(str(date_start)[:10])
    else: 
        click.echo(str(date_start)[:10] + ' to ' + str(date_end)[:10])

    for entry in entries:
        current_level = 0
        current_tags = entry['tag'].split('#')

        for tag in current_tags:
            spaces = ' ' * (current_level * 2)
            if (current_level < len(previous_tags) and tag != previous_tags[current_level]) or (current_level >= len(previous_tags)):
                if tag != '':
                    click.echo(spaces+'  • '+tag)
            
            if tag != '':
                current_level+=1

        spaces = ' ' * (current_level * 2)
        click.echo(spaces + '  • ' + str(entry['entry']))

        previous_tags = current_tags

if __name__ == '__main__':
    cli(obj={})


import sqlite3
import os
from datetime import datetime, date
from dateutil import relativedelta
import click

def init_db(db_path, db_filename):
    # sqlite3 doens't like the ~ alias so we have to expand it first
    home_directory = os.path.expanduser('~')
    expanded_db_path = db_path.replace('~',home_directory)

    # try to make the directory if it doesn't exist
    if not os.path.exists(expanded_db_path):
        os.makedirs(expanded_db_path)

    db_file_path = os.path.join(expanded_db_path,db_filename)

    # connect and create the db if it doesn't exist. create the table if it doesn't exist.
    con = sqlite3.connect(db_file_path)
    con.execute("CREATE TABLE IF NOT EXISTS entry (id INTEGER PRIMARY KEY, insert_date DEFAULT CURRENT_TIMESTAMP, entry TEXT, tag)")

    return con

def last_entry_timestamp(con):
    
    dt_string = "0001-01-01 00:00:00" 
    format = "%Y-%m-%d %H:%M:%S"
    default_entry_timestamp = datetime.strptime(dt_string, format)
    last_entry_timestamp = default_entry_timestamp

    with con:
        cur = con.cursor()
        res = cur.execute("SELECT MAX(insert_date) FROM entry")
        last_entry_timestamp = res.fetchone()[0]

        if last_entry_timestamp==None:
            last_entry_timestamp=default_entry_timestamp
    
    return datetime.strptime(str(last_entry_timestamp), format)

def calculate_date_range(duration,date_start,date_end):
    if date_start==None and date_end:
        click.echo("Must specify --date_start if providing --date_end")
        exit()

    elif (duration and (date_start or date_end)):
        click.echo("Must specify either --duration or --date_start and --date_end")
        exit()

    elif (duration==None and date_start==None and date_end==None):
        #click.echo(f'using default duration {duration}')
        duration="day"
        date_start=datetime.utcnow().date()
        date_end=datetime.utcnow()
        
    elif (duration):
        #click.echo(f'using duration {duration}')
        date_end=datetime.utcnow()

        if duration=="day":
            date_start=datetime.utcnow().date()
        elif duration=="week":
            date_start=datetime.utcnow().date()+relativedelta.relativedelta(weeks=-1)
        elif duration=="month":
            date_start=datetime.utcnow().date()+relativedelta.relativedelta(months=-1)
        elif duration=="year":
            date_start=datetime.utcnow().date()+relativedelta.relativedelta(years=-1)
        else:
            raise Exception("Invalid duration value passed.")

    elif (date_start):
        #click.echo(f'using date_start {date_start}')

        if (date_end):
            #click.echo(f'using date_end {date_end}')
            pass
        else:
            date_end = datetime.utcnow()
            #click.echo(f'using default date_end today {date_end}')
    
    return (date_start,date_end)

def retrieve_entries(date_start,date_end,con):
    with con:
        cur = con.cursor()
        res = cur.execute("""SELECT insert_date,entry,tag 
                            FROM entry 
                            WHERE insert_date >= ?
                            AND insert_date <= ?
                            ORDER BY tag, insert_date""",(date_start,date_end))

        # add column names as keys instead of index numbers
        column_names  = cur.description
        result = [{column_names[index][0]: column for index, column in enumerate(value)} for value in cur.fetchall()]

        return result

class DistinctTags(click.ParamType):
    name = "distinct_tags"

    def shell_complete(self, ctx, param, incomplete):
        con = init_db("~/.jurn","jurn.db")
        with con:
            cur = con.cursor()
            res = cur.execute("""SELECT DISTINCT tag 
                                FROM entry
                                WHERE tag like ? 
                                ORDER BY tag""",(incomplete+'%',))
            results =  list(cur.fetchall())

        return [
            click.shell_completion.CompletionItem(value[0]) for value in results
        ]
